@@ -47,11 +47,12 @@ static struct semaphore user_ls_lock;
 /* Flags in li_flags */
 #define LI_FLAG_COMPLETE  1
 #define LI_FLAG_FIRSTLOCK 2
+#define LI_FLAG_ONLIST    3
 
-
-/* flags in ls_flags*/
+/* flags in ls_flags */
 #define LS_FLAG_DELETED   1
 #define LS_FLAG_AUTOFREE  2
+#define LS_FLAG_DEFAULT   3
 
 
 #define LOCKINFO_MAGIC 0x53595324
@@ -100,6 +101,9 @@ struct user_ls {
 /* misc_device info for the control device */
 static struct miscdevice ctl_device;
 
+/* Bit 1 of this is set when we have a default ls */
+static long default_ls;
+
 /*
  * Stuff we hang off the file struct.
  * The first two are to cope with unlocking all the
@@ -216,6 +220,8 @@ static int register_lockspace(char *name, struct user_ls **ls, int flags)
 
 	if (flags & DLM_USER_LSFLG_AUTOFREE)
 		set_bit(LS_FLAG_AUTOFREE, &newls->ls_flags);
+	if (flags & DLM_USER_LSFLG_DEFAULTLS)
+		set_bit(LS_FLAG_DEFAULT, &newls->ls_flags);
 
 	add_lockspace_to_list(newls);
 	*ls = newls;
@@ -225,25 +231,32 @@ static int register_lockspace(char *name, struct user_ls **ls, int flags)
 /* Called with the user_ls_lock semaphore held */
 static int unregister_lockspace(struct user_ls *lsinfo, int force)
 {
-	int status;
+	int status = 0;
 
-	status = dlm_release_lockspace(lsinfo->ls_lockspace, force);
-	if (status)
-		return status;
-
-	status = misc_deregister(&lsinfo->ls_miscinfo);
-	if (status)
-		return status;
-
-	list_del(&lsinfo->ls_list);
-	set_bit(LS_FLAG_DELETED, &lsinfo->ls_flags);
-	lsinfo->ls_lockspace = NULL;
 	if (atomic_read(&lsinfo->ls_refcnt) == 0) {
+		status = dlm_release_lockspace(lsinfo->ls_lockspace, force);
+		if (status)
+			return status;
+
+		status = misc_deregister(&lsinfo->ls_miscinfo);
+		if (status)
+			return status;
+
+		list_del(&lsinfo->ls_list);
+		set_bit(LS_FLAG_DELETED, &lsinfo->ls_flags);
+		lsinfo->ls_lockspace = NULL;
+		if (test_bit(LS_FLAG_DEFAULT, &lsinfo->ls_flags))
+			clear_bit(1, &default_ls);
+
+
 		kfree(lsinfo->ls_miscinfo.name);
 		kfree(lsinfo);
 	}
+	else {
+		status = -EBUSY;
+	}
 
-	return 0;
+	return status;
 }
 
 /* Add it to userland's AST queue */
@@ -334,13 +347,14 @@ static void ast_routine(void *param)
 			if (!lkb) {
 				spin_lock(&li->li_file->fi_lkb_lock);
 				list_del(&li->li_ownerqueue);
+				clear_bit(LI_FLAG_ONLIST, &li->li_flags);
 				spin_unlock(&li->li_file->fi_lkb_lock);
 				release_lockinfo(li);
 			}
 			return;
 		}
 		/* Free unlocks & queries */
-		if (li->li_lksb.sb_status == -DLM_EUNLOCK ||
+		if ((li->li_lksb.sb_status == -DLM_EUNLOCK && !lkb) ||
 		    li->li_cmd == DLM_USER_QUERY) {
 			release_lockinfo(li);
 		}
@@ -564,6 +578,7 @@ static int dlm_close(struct inode *inode, struct file *file)
 		}
 	}
 	up(&user_ls_lock);
+	put_file_info(f);
 
 	/* Restore signals */
 	sigprocmask(SIG_SETMASK, &tmpsig, NULL);
@@ -612,9 +627,12 @@ static int do_user_create_lockspace(struct file_info *fi, uint8_t cmd,
 	int status;
 	struct user_ls *lsinfo;
 
-	if (!capable(CAP_SYS_ADMIN))
+	if (!capable(CAP_SYS_ADMIN) && !(kparams->flags & DLM_USER_LSFLG_DEFAULTLS))
 		return -EPERM;
 
+	if (kparams->flags & DLM_USER_LSFLG_DEFAULTLS && test_and_set_bit(1, &default_ls))
+		return -EEXIST;
+
 	status = register_lockspace(kparams->name, &lsinfo, kparams->flags);
 
 	/* If it succeeded then return the minor number */
@@ -911,6 +929,7 @@ static int do_user_lock(struct file_info *fi, uint8_t cmd, struct dlm_lock_param
 			if (li) {
 				spin_lock(&fi->fi_lkb_lock);
 				list_add(&li->li_ownerqueue, &fi->fi_lkb_list);
+				set_bit(LI_FLAG_ONLIST, &li->li_flags);
 				spin_unlock(&fi->fi_lkb_lock);
 			}
 			else {
@@ -975,12 +994,16 @@ static int do_user_lock(struct file_info *fi, uint8_t cmd, struct dlm_lock_param
 			   bast_routine : NULL,
 			  kparams->range.ra_end ? &kparams->range : NULL);
 
+	if (status)
+		goto out_err;
+
 	/* If it succeeded (this far) with a new lock then keep track of
 	   it on the file's lkb list */
-	if (!status && !(kparams->flags & DLM_LKF_CONVERT)) {
+	if (!(kparams->flags & DLM_LKF_CONVERT)) {
 
 		spin_lock(&fi->fi_lkb_lock);
 		list_add(&li->li_ownerqueue, &fi->fi_lkb_list);
+		set_bit(LI_FLAG_ONLIST, &li->li_flags);
 		spin_unlock(&fi->fi_lkb_lock);
 
 		up(&li->li_firstlock);
@@ -995,7 +1018,6 @@ static int do_user_lock(struct file_info *fi, uint8_t cmd, struct dlm_lock_param
 		release_lockinfo(li);
 	}
 	return status;
-
 }
 
 static int do_user_unlock(struct file_info *fi, uint8_t cmd, struct dlm_lock_params *kparams)
@@ -1021,6 +1043,7 @@ static int do_user_unlock(struct file_info *fi, uint8_t cmd, struct dlm_lock_par
 		li = allocate_lockinfo(fi, cmd, kparams);
 		spin_lock(&fi->fi_lkb_lock);
 		list_add(&li->li_ownerqueue, &fi->fi_lkb_list);
+		set_bit(LI_FLAG_ONLIST, &li->li_flags);
 		spin_unlock(&fi->fi_lkb_lock);
 	}
 	if (!li)
@@ -1029,6 +1052,12 @@ static int do_user_unlock(struct file_info *fi, uint8_t cmd, struct dlm_lock_par
 	if (li->li_magic != LOCKINFO_MAGIC)
 		return -EINVAL;
 
+	/* Wait until dlm_lock() has completed */
+	if (!test_bit(LI_FLAG_ONLIST, &li->li_flags)) {
+		down(&li->li_firstlock);
+		up(&li->li_firstlock);
+	}
+
 	li->li_user_lksb = kparams->lksb;
 	li->li_castparam = kparams->castparam;
 	li->li_cmd       = cmd;
@@ -1045,6 +1074,7 @@ static int do_user_unlock(struct file_info *fi, uint8_t cmd, struct dlm_lock_par
 	if (!convert_cancel) {
 		spin_lock(&fi->fi_lkb_lock);
 		list_del(&li->li_ownerqueue);
+		clear_bit(LI_FLAG_ONLIST, &li->li_flags);
 		spin_unlock(&fi->fi_lkb_lock);
 	}
 
@@ -1056,6 +1086,7 @@ static int do_user_unlock(struct file_info *fi, uint8_t cmd, struct dlm_lock_par
 		/* It failed, put it back on the list */
 		spin_lock(&fi->fi_lkb_lock);
 		list_add(&li->li_ownerqueue, &fi->fi_lkb_list);
+		set_bit(LI_FLAG_ONLIST, &li->li_flags);
 		spin_unlock(&fi->fi_lkb_lock);
 	}
 
