@@ -535,6 +535,14 @@ do_write_direct_alloc(struct file *file, char *buf, size_t size, loff_t *offset)
 	gfs_quota_unlock_m(ip);
 	gfs_alloc_put(ip);
 
+	if (file->f_mapping->nrpages) {
+		error = filemap_fdatawrite(file->f_mapping);
+		if (!error)
+			error = filemap_fdatawait(file->f_mapping);
+	}
+	if (error)
+		return error;
+
 	return count;
 
  fail_end_trans:
@@ -774,7 +782,14 @@ do_do_write_buf(struct file *file, char *buf, size_t size, loff_t *offset)
 			error = count;
 			goto fail_end_trans;
 		}
-
+		if (gfs_is_stuffed(ip)){
+			struct page *page;
+			page = find_get_page(file->f_mapping, 0);
+			if (page) {
+				ClearPageUptodate(page);
+				page_cache_release(page);
+			}
+		}
 		*offset += count;
 	} else {
 		struct iovec local_iov = { .iov_base = buf, .iov_len = size };
@@ -800,8 +815,14 @@ do_do_write_buf(struct file *file, char *buf, size_t size, loff_t *offset)
 
 	gfs_trans_end(sdp);
 
-	if (file->f_flags & O_SYNC)
+	if (file->f_flags & O_SYNC || IS_SYNC(inode)) {
 		gfs_log_flush_glock(ip->i_gl);
+		error = filemap_fdatawrite(file->f_mapping);
+		if (error == 0)
+			error = filemap_fdatawait(file->f_mapping);
+		if (error)
+			goto fail_ipres;
+	}
 
 	if (alloc_required) {
 		gfs_assert_warn(sdp, count != size ||
@@ -931,12 +952,12 @@ gfs_write(struct file *file, const char *buf, size_t size, loff_t *offset)
 	if (!access_ok(VERIFY_READ, buf, size))
 		return -EFAULT;
 
-	down(&inode->i_sem);
+	mutex_lock(&inode->i_mutex);
 	if (file->f_flags & O_DIRECT)
 		count = walk_vm(file, (char *)buf, size, offset, do_write_direct);
 	else
 		count = walk_vm(file, (char *)buf, size, offset, do_write_buf);
-	up(&inode->i_sem);
+	mutex_unlock(&inode->i_mutex);
 
 	return count;
 }
