@@ -81,6 +81,7 @@ int is_clustered = 1;
 #define ACTION_UNFENCE       8
 #define ACTION_LIST_BANNED   9
 #define ACTION_FAIL_SERVER  10
+#define ACTION_GET_UID      11
 
 gnbd_info_t *match_info_name(char *name){
   list_t *item;
@@ -96,7 +97,8 @@ gnbd_info_t *match_info_name(char *name){
 
 
 
-int read_from_server(char *host, uint32_t request, char **buf)
+int talk_to_server(char *host, uint32_t request, char **buf, void *request_data,
+                     ssize_t request_data_size)
 {
   int sock_fd;
   int n, total;
@@ -110,27 +112,33 @@ int read_from_server(char *host, uint32_t request, char **buf)
   }
   msg = cpu_to_be32(request);
   if (write(sock_fd, &msg, sizeof(msg)) != sizeof(msg)){
-    printe("error sending list request to %s : %s\n", host, strerror(errno));
+    printe("error sending request to %s : %s\n", host, strerror(errno));
     exit(1);
   }
+  if (request_data){
+    if (write(sock_fd, request_data, request_data_size) != request_data_size){
+      printe("error sending request data to %s : %s\n", host, strerror(errno));
+      exit(1);
+    }
+  }
   if (read(sock_fd, &msg, sizeof(msg)) != sizeof(msg)){
     printe("error reading reply from %s : %s\n", host, strerror(errno));
     exit(1);
   }
   msg = cpu_to_be32(msg);
   if (msg != EXTERN_SUCCESS_REPLY){
-    printe("list request to %s failed : %s\n", host, strerror(REPLY_ERR(msg)));
+    printe("request to %s failed : %s\n", host, strerror(REPLY_ERR(msg)));
     exit(1);
   }
   if (read(sock_fd, &msg, sizeof(msg)) != sizeof(msg)){
-    printe("error reading size of list from %s : %s\n", host,
+    printe("error reading size of reply from %s : %s\n", host,
            strerror(errno));
     exit(1);
   }
   msg = be32_to_cpu(msg);
   *buf = malloc(msg);
   if (*buf == NULL){
-    printe("couldn't allocate memory for list : %s\n", strerror(errno));
+    printe("couldn't allocate memory for server reply : %s\n", strerror(errno));
     exit(1);
   }
   memset(*buf, 0, msg);
@@ -138,7 +146,7 @@ int read_from_server(char *host, uint32_t request, char **buf)
   while(total < msg){
     n = read(sock_fd, *buf + total, msg - total);
     if (n <= 0){
-      printe("error reading list from server %s : %s\n", host,
+      printe("error reading reply data from server %s : %s\n", host,
              strerror(errno));
       exit(1);
     }
@@ -148,6 +156,9 @@ int read_from_server(char *host, uint32_t request, char **buf)
   return total;
 }
 
+#define read_from_server(host, request, buf) \
+  talk_to_server(host, request, buf, NULL, 0)
+
 int start_gnbd_monitor(int minor_nr, int timeout, char *host)
 {
   int ret;
@@ -260,7 +271,8 @@ int usage(void){
 "  -r [GNBD | list] remove specified GNBD(s)\n"
 "  -s <host>        IO fence the host (from known servers by default)\n"
 "  -t <server>      specify a server for the IO fence (with -s or -u)\n"
-"  -u <host>        unfence the host (from known servers by default)\n" 
+"  -u <host>        unfence the host (from known servers by default)\n"
+"  -U <GNBD>        get the Universal Identifier for the specified GNBD\n"
 "  -V               version information\n"
 "  -v               verbose output\n"
 "\n");
@@ -586,6 +598,70 @@ void remove_gnbd(char *name, int minor, int pid)
   close(fd);
 }
 
+void kill_gnbd_monitor(void){
+  int pid;
+  if (check_lock("gnbd_monitor.pid", &pid)){
+    if (pid > 0){
+      kill(pid, SIGKILL);
+    }
+  }
+}
+
+int device_active(int minor){
+  list_t *item;
+  gnbd_info_t *gnbd_info;
+  char prefix[] = "/dev/gnbd/";
+  char dev_path[256];
+  
+  list_foreach(item, &gnbd_list){
+    struct stat statbuf;
+    gnbd_info = list_entry(item, gnbd_info_t, list);
+    if ( gnbd_info->minor_nr == minor ){
+      strcpy(dev_path, prefix);
+      strcat(dev_path, gnbd_info->name);
+      
+      if ( lstat(dev_path, &statbuf) == 0 ){
+	if ( S_ISBLK(statbuf.st_mode) )
+	  return 1; /* this is a valid block device */
+      } else {
+	if ( errno != ENOENT ){
+	  printe("failed to stat device %s : %s\n", dev_path, strerror(errno));
+	  return 1; /* if stat on the device failed for some reason*/
+	}
+      }
+    }
+  }
+  return 0;
+}
+
+int uncached_imports_removed(){
+  if (check_lock("gnbd_monitor.pid", NULL)){
+    monitor_info_t *ptr, *devs;
+    int i, count;
+    
+    if (do_list_monitored_devs(&devs, &count) < 0){
+      printe("unable to get monitored device list : %s\n", gstrerror(errno));
+      return 1;
+    }
+    if ( count <= 0 ){ /* monitoring nothing */
+      free(devs);
+      return 1;
+    }
+    
+    ptr = devs;
+    for (i = 0; i < count; i++, ptr++){
+      if ( device_active(ptr->minor_nr) ){
+	/* found an active device, bail */
+	free(devs);
+	return 0;
+      }
+    }
+    free(devs);
+    return 1; /* No active devices found */
+  }
+  return 0;
+}
+
 void update_devs(void){
   int minor;
   char path[LINE_MAX];
@@ -624,7 +700,9 @@ void update_devs(void){
           remove_gnbd(NULL, minor, info->pid);
           printm("removed duplicate gnbd device %s\n", info->name);
           free(info);
-        }
+	  if (uncached_imports_removed())
+	    kill_gnbd_monitor();
+	}
         else {
           printe("cannot create %s for gnbd device minor #%d, "
                  "name already used\n", path, minor);
@@ -671,12 +749,16 @@ void remove_all(void){
     gnbd_info = list_entry(item, gnbd_info_t, list);
     remove_gnbd(gnbd_info->name, gnbd_info->minor_nr, gnbd_info->pid);
   }
+
+  if (uncached_imports_removed())
+    kill_gnbd_monitor();
 }
 
 void remove_clients(char *names[], int num){
   list_t *item;
   gnbd_info_t *gnbd_info;
   int i;
+
   for (i = 0; i < num; i++){
     list_foreach(item, &gnbd_list){
       gnbd_info = list_entry(item, gnbd_info_t, list);
@@ -690,6 +772,9 @@ void remove_clients(char *names[], int num){
       exit(1);
     }
   }
+
+  if (uncached_imports_removed())
+    kill_gnbd_monitor();
 }
 
 void fence(char *host, char *server, int is_fence)
@@ -953,7 +1038,7 @@ void list(void){
     printf("Device name : %s\n"
            "----------------------\n"
            "    Minor # : %d\n"
-           "  Proc name : /dev/gnbd%d\n"
+           " sysfs name : /block/gnbd%d\n"
            "     Server : %s\n"
            "       Port : %d\n"
            "      State : %s %s %s\n"
@@ -972,6 +1057,33 @@ void list(void){
   }
 }
 
+void get_uid(char *name){
+  list_t *item;
+  gnbd_info_t *gnbd = NULL;
+  char *uid;
+  device_req_t req;
+  int minor_nr = -1;
+
+  sscanf(name, "/block/gnbd%d", &minor_nr);
+  list_foreach(item, &gnbd_list){
+    gnbd = list_entry(item, gnbd_info_t, list);
+    if (minor_nr >= 0 && minor_nr == gnbd->minor_nr);
+      break;
+    if (strncmp(gnbd->name, name, 32) == 0)
+      break;
+    gnbd = NULL;
+  }
+  if (!gnbd){
+    printe("cannot find device '%s'\n", name);
+    exit(1);
+  }
+  strncpy(req.name, gnbd->name, 32);
+  req.name[31] = 0;
+  talk_to_server(gnbd->server_name, EXTERN_UID_REQ, &uid, &req, sizeof(req));
+  printf("%s\n", uid);
+  free(uid);
+}
+  
 void validate_gnbds(void){
   list_t *item;
   char filename[32];
@@ -1000,18 +1112,18 @@ void validate_gnbds(void){
     }
   }
 }
-        
 
 int main(int argc, char **argv)
 {
   int action = 0;
   char *host = NULL;
   char *fence_server = NULL;
+  char *dev = NULL;
   int c;
 
   list_init(&gnbd_list);
   program_name = "gnbd_import";
-  while( (c = getopt(argc, argv, "ac:e:hi:lnOp:qRrs:t:u:Vv")) != -1){
+  while( (c = getopt(argc, argv, "ac:e:hi:lnOp:qRrs:t:U:u:Vv")) != -1){
     switch(c){
     case ':':
     case '?':
@@ -1073,6 +1185,10 @@ int main(int argc, char **argv)
     case 't':
       fence_server = optarg;
       continue;
+    case 'U':
+      set_action(ACTION_GET_UID);
+      dev = optarg;
+      continue;
     case 'u':
       set_action(ACTION_UNFENCE);
       host = optarg;
@@ -1091,7 +1207,7 @@ int main(int argc, char **argv)
       continue;
     default:
       printe("invalid option -- %c\n", c);
-      printf("Please use '-h' for usage.\n");
+      fprintf(stderr, "Please use '-h' for usage.\n");
       exit(1);
     }
   }
@@ -1164,6 +1280,9 @@ int main(int argc, char **argv)
   case ACTION_LIST_BANNED:
     get_banned_list(host);
     return 0;
+  case ACTION_GET_UID:
+    get_uid(dev);
+    return 0;
   default:
     printe("unknown action %d\n", action);
     return 1;
