@@ -72,7 +72,7 @@ count_resource_groups(uint64_t nodeid, int *excl)
 		*excl = 0;
 
 	list_do(&_resources, res) {
-		if (res->r_rule->rr_type == 0)
+		if (res->r_rule->rr_root == 0)
 			continue;
 
 		rgname = res->r_attrs[0].ra_value;
@@ -539,16 +539,17 @@ send_rg_state(int fd, char *rgname)
 
 
 /**
-  Send all resource group states to a file descriptor
-
-  @param fd		File descriptor to send states to.
-  @return		0
+  Send status from a thread because we don't want rgmanager's
+  main thread to block in the case of DLM issues
  */
-int
-send_rg_states(int fd)
+static void *
+status_check_thread(void *arg)
 {
+	int fd = *(int *)arg;
 	resource_t *res;
 
+	free(arg);
+
 	pthread_rwlock_rdlock(&resource_lock);
 
 	list_do(&_resources, res) {
@@ -563,6 +564,39 @@ send_rg_states(int fd)
 	msg_send_simple(fd, RG_SUCCESS, 0, 0);
 	msg_close(fd);
 
+	return NULL;
+}
+
+
+/**
+  Send all resource group states to a file descriptor
+
+  @param fd		File descriptor to send states to.
+  @return		0
+ */
+int
+send_rg_states(int fd)
+{
+	int *fdp;
+	pthread_t newthread;
+	pthread_attr_t attrs;
+
+	fdp = malloc(sizeof(int));
+	if (!fdp) {
+		msg_send_simple(fd, RG_FAIL, 0, 0);
+		return -1;
+	}
+
+	*fdp = fd;
+
+        pthread_attr_init(&attrs);
+        pthread_attr_setinheritsched(&attrs, PTHREAD_INHERIT_SCHED);
+        pthread_attr_setdetachstate(&attrs, PTHREAD_CREATE_DETACHED);
+	pthread_attr_setstacksize(&attrs, 65535);
+
+	pthread_create(&newthread, &attrs, status_check_thread, fdp);
+        pthread_attr_destroy(&attrs);
+
 	return 0;
 }
 
@@ -653,7 +687,7 @@ do_status_checks(void)
 		    svcblk.rs_state != RG_STATE_STARTED)
 			continue;
 
-		clulog(LOG_DEBUG, "Checking status of %s\n", name);
+		/*clulog(LOG_DEBUG, "Checking status of %s\n", name);*/
 
 		rt_enqueue_request(name, RG_STATUS,
 				   -1, 0, NODE_ID_NONE, 0, 0);
@@ -661,7 +695,7 @@ do_status_checks(void)
 	} while (!list_done(&_tree, curr));
 
 	pthread_rwlock_unlock(&resource_lock);
-	rg_wait_threads();
+	/*rg_wait_threads();*/
 }
 
 /**
@@ -734,7 +768,7 @@ do_condstarts(void)
 	pthread_rwlock_rdlock(&resource_lock);
 	list_do(&_tree, curr) {
 
-		if (curr->rn_resource->r_rule->rr_type == 0)
+		if (curr->rn_resource->r_rule->rr_root == 0)
 			continue;
 
 		/* Group name */
@@ -779,7 +813,7 @@ do_condstarts(void)
 	pthread_rwlock_rdlock(&resource_lock);
 	list_do(&_tree, curr) {
 
-		if (curr->rn_resource->r_rule->rr_type == 0)
+		if (curr->rn_resource->r_rule->rr_root == 0)
 			continue;
 
 		/* Group name */
@@ -825,7 +859,7 @@ int
 check_config_update(void)
 {
 	int newver = 0, fd, ret = 0;
-	char *val;
+	char *val = NULL;
 
        	fd = ccs_lock();
 	if (fd == -1) {
@@ -836,6 +870,9 @@ check_config_update(void)
 		newver = atoi(val);
 	}
 
+	if (val)
+		free(val);
+
 	pthread_mutex_lock(&config_mutex);
 	if (newver && newver != config_version)
 		ret = 1;
