@@ -34,6 +34,7 @@
 #include <rg_queue.h>
 #include <malloc.h>
 
+int configure_logging(int ccsfd);
 
 int daemon_init(char *);
 int init_resource_groups(int);
@@ -45,10 +46,11 @@ void flag_shutdown(int sig);
 void hard_exit(void);
 int send_rg_states(int);
 int check_config_update(void);
+int svc_exists(char *);
 
 int shutdown_pending = 0, running = 1, need_reconfigure = 0;
-
-#define request_failback(a) 0
+char debug = 0; /* XXX* */
+static int signalled = 0;
 
 uint64_t next_node_id(cluster_member_list_t *membership, uint64_t me);
 
@@ -112,35 +114,6 @@ notify_exiting(void)
 }
 
 
-int
-request_failbacks(void)
-{
-	int x;
-	uint64_t partner;
-	cluster_member_list_t *membership;
-
-	membership = member_list();
-
-	for (x = 0; x < membership->cml_count; x++) {
-
-		partner = membership->cml_members[x].cm_id;
-
-		if (partner == my_id() ||
-		    membership->cml_members[x].cm_state != STATE_UP)
-			continue;
-
-		if (request_failback(partner) != 0) {
-			clulog(LOG_ERR, "#35: Unable to inform partner "
-			       "to start failback\n");
-		}
-	}
-
-	cml_free(membership);
-
-	return 0;
-}
-
-
 void
 flag_reconfigure(int sig)
 {
@@ -336,7 +309,7 @@ dispatch_msg(int fd, uint64_t nodeid)
 		swab_SmMessageSt(&msg_sm);
 
 		if (rg_locked()) {
-			msg_sm.sm_data.d_ret = -EAGAIN;
+			msg_sm.sm_data.d_ret = RG_EAGAIN;
 			/* Encode before responding... */
 			swab_SmMessageSt(&msg_sm);
 
@@ -346,6 +319,17 @@ dispatch_msg(int fd, uint64_t nodeid)
 				       "action request.\n");
 		}
 
+		if (!svc_exists(msg_sm.sm_data.d_svcName)) {
+			msg_sm.sm_data.d_ret = RG_ENOSERVICE;
+			/* No such service! */
+			swab_SmMessageSt(&msg_sm);
+
+			if (msg_send(fd, &msg_sm, sizeof (SmMessageSt)) !=
+		    	    sizeof (SmMessageSt))
+				clulog(LOG_ERR, "#40: Error replying to "
+				       "action request.\n");
+		}
+
 		/* Queue request */
 		rt_enqueue_request(msg_sm.sm_data.d_svcName,
 		  		   msg_sm.sm_data.d_action,
@@ -425,6 +409,8 @@ handle_cluster_event(int fd)
 }
 
 
+void dump_threads(void);
+
 int
 event_loop(int clusterfd)
 {
@@ -436,6 +422,14 @@ event_loop(int clusterfd)
 	tv.tv_sec = 10;
 	tv.tv_usec = 0;
 
+	if (signalled) {
+		signalled = 0;
+		/*
+		malloc_stats();
+		dump_threads();
+		 */
+	}
+
 	while (tv.tv_sec || tv.tv_usec) {
 		FD_ZERO(&rfds);
 		max = msg_fill_fdset(&rfds, MSG_LISTEN, RG_PURPOSE);
@@ -480,6 +474,7 @@ event_loop(int clusterfd)
 
 	if (need_reconfigure || check_config_update()) {
 		need_reconfigure = 0;
+		configure_logging(-1);
 		init_resource_groups(1);
 		return 0;
 	}
@@ -538,17 +533,48 @@ cleanup(int cluster_fd)
 }
 
 
-void dump_threads(void);
 
 void
 statedump(int sig)
 {
-	dump_threads();
-	/*malloc_stats();*/
+	signalled++;
 }
 
 
-void malloc_dump_table(void);
+void malloc_dump_table(size_t, size_t);
+
+
+/*
+ * Configure logging based on data in cluster.conf
+ */
+int
+configure_logging(int ccsfd)
+{
+	char *v;
+	char internal = 0;
+
+	if (ccsfd == -1) {
+		internal = 1;
+		ccsfd = ccs_connect();
+		if (ccsfd == -1)
+			return -1;
+	}
+
+	if (ccs_get(ccsfd, "/cluster/rm/@log_facility", &v) == 0) {
+		clu_set_facility(v);
+		free(v);
+	}
+
+	if (ccs_get(ccsfd, "/cluster/rm/@log_level", &v) == 0) {
+		clu_set_loglevel(atoi(v));
+		free(v);
+	}
+
+	if (internal)
+		ccs_disconnect(ccsfd);
+
+	return 0;
+}
 
 
 void
@@ -597,7 +623,7 @@ int
 main(int argc, char **argv)
 {
 	int cluster_fd, rv;
-	char debug = 0, foreground = 0;
+	char foreground = 0;
 	int quorate;
 	int listen_fds[2], listeners;
 	uint64_t myNodeID;
@@ -635,6 +661,9 @@ main(int argc, char **argv)
 	   We know we're quorate.  At this point, we need to
 	   read the resource group trees from ccsd.
 	 */
+	configure_logging(-1);
+	clulog(LOG_NOTICE, "Resource Group Manager Starting\n");
+
 	if (init_resource_groups(0) != 0) {
 		clulog(LOG_CRIT, "#8: Couldn't initialize services\n");
 		return -1;
@@ -658,7 +687,11 @@ main(int argc, char **argv)
 	setup_signal(SIGUSR1, statedump);
 	unblock_signal(SIGCHLD);
 	setup_signal(SIGPIPE, SIG_IGN);
-	setup_signal(SIGSEGV, segfault);
+	if (debug) {
+		setup_signal(SIGSEGV, segfault);
+	} else {
+		unblock_signal(SIGSEGV);
+	}
 
 	if ((listeners = msg_listen(RG_PORT, RG_PURPOSE,
 				    listen_fds, 2)) <= 0) {
@@ -723,6 +756,7 @@ main(int argc, char **argv)
 	
 	/*malloc_dump_table(); */ /* Only works if alloc.c us used */
 	/*malloc_stats();*/
+	/*malloc_dump_table(1352, 1352);*/
 
 	exit(0);
 }
