@@ -32,6 +32,11 @@
 #include <clulog.h>
 #include <assert.h>
 
+#ifdef INTERNAL_MALLOC
+void malloc_zap_mutex(void);
+#endif
+
+
 /* XXX from resrules.c */
 int store_childtype(resource_child_t **childp, char *name, int start,
 		    int stop, int forbid);
@@ -221,10 +226,10 @@ build_env(resource_node_t *node, int op, int depth)
 	resource_t *res = node->rn_resource;
 	char **env;
 	char *val;
-	int x, n;
+	int x, attrs, n;
 
-	for (x = 0; res->r_attrs && res->r_attrs[x].ra_name; x++);
-	x += 7; /*
+	for (attrs = 0; res->r_attrs && res->r_attrs[attrs].ra_name; attrs++);
+	attrs += 7; /*
 		   Leave space for:
 		   OCF_RA_VERSION_MAJOR
 		   OCF_RA_VERSION_MINOR
@@ -235,12 +240,14 @@ build_env(resource_node_t *node, int op, int depth)
 		   (null terminator)
 		 */
 
-	env = malloc(sizeof(char *) * x);
+	env = malloc(sizeof(char *) * attrs);
 	if (!env)
 		return NULL;
 
-	memset(env, 0, sizeof(char *) * x);
+	memset(env, 0, sizeof(char *) * attrs);
 
+	/* Reset */
+	attrs = 0;
 	for (x = 0; res->r_attrs && res->r_attrs[x].ra_name; x++) {
 
 		val = attr_value(node, res->r_attrs[x].ra_name);
@@ -251,24 +258,25 @@ build_env(resource_node_t *node, int op, int depth)
 		n = strlen(res->r_attrs[x].ra_name) + strlen(val) + 2 +
 			strlen(OCF_RES_PREFIX);
 
-		env[x] = malloc(n);
-		if (!env[x]) {
+		env[attrs] = malloc(n);
+		if (!env[attrs]) {
 			kill_env(env);
 			return NULL;
 		}
 
 		/* Prepend so we don't conflict with normal shell vars */
-		snprintf(env[x], n, "%s%s=%s", OCF_RES_PREFIX,
+		snprintf(env[attrs], n, "%s%s=%s", OCF_RES_PREFIX,
 			 res->r_attrs[x].ra_name, val);
-		
+
 #if 0
 		/* Don't uppercase; OCF-spec */
 		for (n = 0; env[x][n] != '='; n++)
 			env[x][n] &= ~0x20; /* Convert to uppercase */
 #endif
+		++attrs;
 	}
 
-	add_ocf_stuff(res, &env[x], depth);
+	add_ocf_stuff(res, &env[attrs], depth);
 
 	return env;
 }
@@ -341,6 +349,9 @@ res_exec(resource_node_t *node, int op, int depth)
 
 	if (!childpid) {
 		/* Child */ 
+#ifdef INTERNAL_MALLOC
+		malloc_zap_mutex();
+#endif
 #if 0
 		printf("Exec of script %s, action %s type %s\n",
 			res->r_rule->rr_agent, res_ops[op],
@@ -660,33 +671,14 @@ print_resource_tree(resource_node_t **tree)
 }
 
 
-/**
-   Nasty codependent function.  Perform an operation by numerical level
-   at some point in the tree.  This allows indirectly-dependent resources
-   (such as IP addresses and user scripts) to have ordering without requiring
-   a direct dependency.
-
-   @param tree		Resource tree to search/perform operations on
-   @param first		Resource we're looking to perform the operation on,
-   			if one exists.
-   @param ret		Unused, but will be used to store status information
-   			such as resources consumed, etc, in the future.
-   @param op		Operation to perform if either first is found,
-   			or no first is declared (in which case, all nodes
-			in the subtree).
-   @see			_res_op res_exec
- */
-int
-_res_op_by_level(resource_node_t **tree, resource_t *first, void *ret,
+static inline int
+_do_child_levels(resource_node_t **tree, resource_t *first, void *ret,
 		 int op)
 {
 	resource_node_t *node = *tree;
 	resource_t *res = node->rn_resource;
 	resource_rule_t *rule = res->r_rule;
-	int l, x, rv = 0, lev;
-
-	if (!rule->rr_childtypes)
-		return _res_op(&node->rn_child, first, NULL, ret, op);
+	int l, lev, x, rv = 0;
 
 	for (l = 1; l <= RESOURCE_MAX_LEVELS; l++) {
 
@@ -720,6 +712,19 @@ _res_op_by_level(resource_node_t **tree, resource_t *first, void *ret,
 			return rv;
 	}
 
+	return 0;
+}
+
+
+static inline int
+_do_child_default_level(resource_node_t **tree, resource_t *first,
+			void *ret, int op)
+{
+	resource_node_t *node = *tree;
+	resource_t *res = node->rn_resource;
+	resource_rule_t *rule = res->r_rule;
+	int x, rv = 0, lev;
+
 	for (x = 0; rule->rr_childtypes &&
 	     rule->rr_childtypes[x].rc_name; x++) {
 
@@ -749,6 +754,57 @@ _res_op_by_level(resource_node_t **tree, resource_t *first, void *ret,
 }
 
 
+
+
+/**
+   Nasty codependent function.  Perform an operation by numerical level
+   at some point in the tree.  This allows indirectly-dependent resources
+   (such as IP addresses and user scripts) to have ordering without requiring
+   a direct dependency.
+
+   @param tree		Resource tree to search/perform operations on
+   @param first		Resource we're looking to perform the operation on,
+   			if one exists.
+   @param ret		Unused, but will be used to store status information
+   			such as resources consumed, etc, in the future.
+   @param op		Operation to perform if either first is found,
+   			or no first is declared (in which case, all nodes
+			in the subtree).
+   @see			_res_op res_exec
+ */
+int
+_res_op_by_level(resource_node_t **tree, resource_t *first, void *ret,
+		 int op)
+{
+	resource_node_t *node = *tree;
+	resource_t *res = node->rn_resource;
+	resource_rule_t *rule = res->r_rule;
+	int rv = 0;
+
+	if (!rule->rr_childtypes)
+		return _res_op(&node->rn_child, first, NULL, ret, op);
+
+	if (op == RS_START || op == RS_STATUS) {
+		rv =  _do_child_levels(tree, first, ret, op);
+	       	if (rv != 0)
+			return rv;
+
+		/* Start default level after specified ones */
+		rv =  _do_child_default_level(tree, first, ret, op);
+
+	} /* stop */ else {
+
+		rv =  _do_child_default_level(tree, first, ret, op);
+	       	if (rv != 0)
+			return rv;
+
+		rv =  _do_child_levels(tree, first, ret, op);
+	}
+
+	return rv;
+}
+
+
 /**
    Do a status on a resource node.  This takes into account the last time the
    status operation was run and selects the highest possible resource depth
