@@ -110,6 +110,75 @@ svc_report_failure(char *svcName)
 }
 
 
+int
+clu_lock_verbose(char *resource, int dflt_flags, void **lockpp)
+{
+	int ret, timed_out = 0;
+	struct timeval start, now;
+	uint64_t nodeid, *p;
+	int flags;
+	int block = !(dflt_flags & CLK_NOWAIT);
+
+	/* Holder not supported for this call */
+	dflt_flags &= ~CLK_HOLDER;
+
+	flags = dflt_flags;
+
+	if (block) {
+		gettimeofday(&start, NULL);
+		start.tv_sec += 30;
+	}
+	while (1) {
+		if (block) {
+			gettimeofday(&now, NULL);
+
+			if ((now.tv_sec > start.tv_sec) ||
+			    ((now.tv_sec == start.tv_sec) &&
+	 		     (now.tv_usec >= start.tv_usec))) {
+
+				gettimeofday(&start, NULL);
+				start.tv_sec += 30;
+
+				timed_out = 1;
+				flags |= CLK_HOLDER;
+			}
+		}
+
+		ret = clu_lock(resource, flags | CLK_NOWAIT, lockpp);
+
+		if ((ret != 0) && (errno == EAGAIN) && block) {
+			if (timed_out) {
+				p = (uint64_t *)*lockpp;
+				if (p) {
+					nodeid = *p;
+					clulog(LOG_WARNING, "Node ID:%08x%08x"
+					       " stuck with lock %s\n",
+					       (uint32_t)(nodeid>>32&0xffffffff),
+					       (uint32_t)nodeid&0xffffffff,
+					       resource);
+					free(p);
+				} else {
+					clulog(LOG_WARNING, "Starving for lock"
+					       " %s\n", resource);
+				}
+				flags = dflt_flags;
+				timed_out = 0;
+			}
+			usleep(random()&32767<<1);
+			continue;
+
+		} else if (ret == 0) {
+			/* Success */
+			return 0;
+		}
+
+		break;
+	}
+
+	return ret;
+}
+
+
 int
 #ifdef DEBUG
 _rg_lock(char *name, void **p)
@@ -117,29 +186,10 @@ _rg_lock(char *name, void **p)
 rg_lock(char *name, void **p)
 #endif
 {
-#if 0
-	int ret;
-#endif
 	char res[256];
 
 	snprintf(res, sizeof(res), "usrm::rg=\"%s\"", name);
-	*p = NULL;
-
-#if 0
-	do {
-		ret = clu_lock(res, CLK_EX | CLK_NOWAIT, p);
-		if ((ret == -1) && (errno == EAGAIN)) {
-			usleep(50000);
-			continue;
-		}
-		
-	} while (1);
-
-	return ret;
-#else
-	return clu_lock(res, CLK_EX, p);
-#endif
-	
+	return clu_lock_verbose(res, CLK_EX, p);
 }
 
 
@@ -1044,29 +1094,31 @@ handle_relocate_req(char *svcName, int request, uint64_t preferred_target,
 			return RG_EFORWARD;
 	}
 
-	allowed_nodes = member_list();
-	/*
-	   Mark everyone except me and the preferred target DOWN for now
-	   If we can't start it on the preferred target, then we'll try
- 	   other nodes.
-	 */
-	for (x = 0; x < allowed_nodes->cml_count; x++) {
-		if (allowed_nodes->cml_members[x].cm_id == me ||
-		    allowed_nodes->cml_members[x].cm_id == preferred_target)
-			continue;
+	if (preferred_target != NODE_ID_NONE) {
 
-		allowed_nodes->cml_members[x].cm_state = STATE_DOWN;
-	}
+		allowed_nodes = member_list();
+		/*
+	   	   Mark everyone except me and the preferred target DOWN for now
+		   If we can't start it on the preferred target, then we'll try
+	 	   other nodes.
+		 */
+		for (x = 0; x < allowed_nodes->cml_count; x++) {
+			if (allowed_nodes->cml_members[x].cm_id == me ||
+		    	    allowed_nodes->cml_members[x].cm_id == preferred_target)
+				continue;
+			allowed_nodes->cml_members[x].cm_state = STATE_DOWN;
+		}
 
-	/*
-	 * First, see if it's legal to relocate to the target node.  Legal
-	 * means: the node is online and is in the [restricted] failover
-	 * domain of the service, or the service has no failover domain.
-	 */
-	if (preferred_target != (uint64_t)FAIL) {
+		/*
+		 * First, see if it's legal to relocate to the target node.  Legal
+		 * means: the node is online and is in the [restricted] failover
+		 * domain of the service, or the service has no failover domain.
+		 */
 
 		target = best_target_node(allowed_nodes, me, svcName, 1);
 
+		cml_free(allowed_nodes);
+
 		/*
 		 * I am the ONLY one capable of running this service,
 		 * PERIOD...
@@ -1074,6 +1126,7 @@ handle_relocate_req(char *svcName, int request, uint64_t preferred_target,
 		if (target == me && me != preferred_target)
 			goto exhausted;
 
+
 		if (target == me) {
 			/*
 			   Relocate to self.  Don't send a network request
@@ -1102,14 +1155,10 @@ handle_relocate_req(char *svcName, int request, uint64_t preferred_target,
 	 * Ok, so, we failed to send it to the preferred target node.
 	 * Try to start it on all other nodes.
 	 */
-	for (x = 0; x < allowed_nodes->cml_count; x++) {
-		if (allowed_nodes->cml_members[x].cm_id == me ||
-		    allowed_nodes->cml_members[x].cm_id == preferred_target) {
-			allowed_nodes->cml_members[x].cm_state = STATE_DOWN;
-			continue;
-		}
-		allowed_nodes->cml_members[x].cm_state = STATE_UP;
-	}
+	allowed_nodes = member_list();
+
+	if (preferred_target != NODE_ID_NONE)
+		memb_mark_down(allowed_nodes, preferred_target);
 	memb_mark_down(allowed_nodes, me);
 
 	while (memb_count(allowed_nodes)) {
@@ -1128,6 +1177,10 @@ handle_relocate_req(char *svcName, int request, uint64_t preferred_target,
 		case NO:
 			/* state uncertain */
 			cml_free(allowed_nodes);
+			clulog(LOG_DEBUG, "State Uncertain: svc:%s "
+			       "nid:%08x%08x req:%d\n", svcName,
+			       (uint32_t)(target>>32)&0xffffffff,
+			       (uint32_t)(target&0xffffffff), request);
 			return 0;
 		case 0:
 			*new_owner = target;
@@ -1196,6 +1249,7 @@ handle_start_req(char *svcName, int req, uint64_t *new_owner)
 		tolerance = FOD_GOOD;
 	
 	if (req != RG_RESTART &&
+	    req != RG_START_RECOVER &&
 	    (node_should_start_safe(my_id(), membership, svcName) <
 	     tolerance)) {
 		cml_free(membership);
@@ -1229,7 +1283,7 @@ handle_start_req(char *svcName, int req, uint64_t *new_owner)
 	clulog(LOG_DEBUG, "Stopping failed service %s\n", svcName);
 	if (svc_stop(svcName, RG_STOP_RECOVER) != 0) {
 		clulog(LOG_CRIT,
-		       "#13: Service %s failed to stop cleanly",
+		       "#13: Service %s failed to stop cleanly\n",
 		       svcName);
 		(void) svc_fail(svcName);
 
